import os
import shutil
import oommfc as oc
import discretisedfield as df


class TestDemag:
    def test_script(self):
        demag = oc.Demag()
        assert 'asymptotic_radius' not in demag._script

        demag = oc.Demag(asymptotic_radius=5)
        assert 'asymptotic_radius' in demag._script

    def test_demag(self):
        p1 = (-10e-9, 0, 0)
        p2 = (10e-9, 1e-9, 1e-9)
        cell = (1e-9, 1e-9, 1e-9)
        mesh = oc.Mesh(p1=p1, p2=p2, cell=cell)

        name = 'demag'
        if os.path.exists(name):
            shutil.rmtree(name)

        Ms = 1e6

        def norm_fun(pos):
            x, y, z = pos
            if -9e-9 <= x <= 9e-9:
                return 0
            else:
                return Ms

        system = oc.System(name=name)
        system.hamiltonian = oc.Demag()
        system.m = df.Field(mesh, dim=3, value=(0, 0, 1), norm=norm_fun)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m((-9.5e-9, 0.5e-9, 0.5e-9))
        value = system.m((9.5e-9, 0.5e-9, 0.5e-9))

        system.delete()

    def test_demag_asymptotic_radius(self):
        p1 = (0, 0, 0)
        p2 = (3e-9, 6e-9, 4e-9)
        cell = (1e-9, 2e-9, 4e-9)
        mesh = oc.Mesh(p1=p1, p2=p2, cell=cell)

        name = 'demag_asymptotic_radius'
        if os.path.exists(name):
            shutil.rmtree(name)

        Ms = 1e6

        system = oc.System(name=name)
        system.hamiltonian = oc.Demag(asymptotic_radius=6)
        system.m = df.Field(mesh, dim=3, value=(0, 0, 1), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        system.delete()



import time
import oommfc as oc
import discretisedfield as df


def macrospin():
    """Return a sytsem that represents a macrospin.

    """
    p1 = (0, 0, 0)
    p2 = (5e-9, 5e-9, 5e-9)
    cell = (5e-9, 5e-9, 5e-9)
    mesh = oc.Mesh(p1=p1, p2=p2, cell=cell)

    system = oc.System(name='example-macrospin')
    system.hamiltonian = oc.Zeeman(H=(0, 0, 5e6))
    system.m = df.Field(mesh, value=(0, 0, 1), norm=8e6)
    system.dynamics = oc.Precession(gamma=oc.consts.gamma) + oc.Damping(alpha=0.05)

    return system



import oommfc as oc

def test_version():
    assert isinstance(oc.__version__, str)
    assert '.' in oc.__version__

def test_dependencies():
    assert isinstance(oc.__dependencies__, list)
    assert len(oc.__dependencies__) > 0
    



import os
import shutil
import numpy as np
import oommfc as oc
import discretisedfield as df


class TestHamiltonian:
    def setup(self):
        self.p1 = (0, 0, 0)
        self.p2 = (10e-9, 5e-9, 3e-9)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.regions = {'r1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 3e-9)),
                        'r2': df.Region(p1=(5e-9, 0, 0), p2=(10e-9, 5e-9, 3e-9))}

    def test_exchange_zeeman(self):
        name = 'hm_exchange_zeeman'
        if os.path.exists(name):
            shutil.rmtree(name)

        A = 1e-12
        H = (1e6, 0, 0)
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell)

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A) + oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(0, 1, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        system.delete()

    def test_exchange_uniaxialanisotropy(self):
        name = 'hm_exchange_uniaxialanisotropy'
        if os.path.exists(name):
            shutil.rmtree(name)

        A = {'r1': 1e-12, 'r2': 0}
        K1 = 1e5
        u = (1, 0, 0)
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell,
                       regions=self.regions)

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A) + oc.UniaxialAnisotropy(K1=K1, u=u)
        system.m = df.Field(mesh, dim=3, value=(0.5, 1, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        system.delete()

    def test_exchange_cubicanisotropy(self):
        name = 'hm_exchange_cubicanisotropy'
        if os.path.exists(name):
            shutil.rmtree(name)

        A = {'r1': 1e-12, 'r2': 0}
        K1 = 1e5
        u1 = (1, 0, 0)
        u2 = (0, 1, 0)
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell,
                       regions=self.regions)

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A) + oc.CubicAnisotropy(K1=K1, u1=u1, u2=u2)
        system.m = df.Field(mesh, dim=3, value=(1, 0.3, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        system.delete()

    def test_exchange_dmi_zeeman(self):
        name = 'hm_exchange_dmi_zeeman'
        if os.path.exists(name):
            shutil.rmtree(name)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell,
                       regions=self.regions)

        # Very weak DMI and strong Zeeman to make the final
        # magnetisation uniform.
        A = {'r1': 1e-12, 'r2': 3e-12, 'r1:r2': 2e-12}
        D = {'r1': 1e-9, 'r2': 0, 'r1:r2': 5e-9}
        H = df.Field(mesh, dim=3, value=(1e10, 0, 0))
        Ms = 1e6

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A) + oc.DMI(D=D, crystalclass='Cnv') + \
            oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(1, 0.3, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1

        system.delete()

    def test_exchange_dmi_zeeman_uniaxialanisotropy_demag(self):
        name = 'exchange_dmi_zeeman_uniaxialanisotropy'
        if os.path.exists(name):
            shutil.rmtree(name)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell,
                       regions=self.regions)

        # Very weak DMI and strong Zeeman to make the final
        # magnetisation uniform.
        A = {'r1': 1e-12, 'r2': 3e-12, 'r1:r2': 2e-12}
        D = {'r1': 1e-9, 'r2': 0, 'r1:r2': 5e-9}  # Very weak DMI
        H = df.Field(mesh, dim=3, value=(1e12, 0, 0))
        K1 = 1e6
        u = (1, 0, 0)
        Ms = 1e5

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A) + oc.DMI(D=D, crystalclass='Cnv') + \
            oc.UniaxialAnisotropy(K1=K1, u=u) + oc.Zeeman(H=H) + oc.Demag()
        system.m = df.Field(mesh, dim=3, value=(1, 0.3, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1

        system.delete()








import os
import discretisedfield as df
import oommfc as oc


class TestDriver:
    def setup(self):
        self.system = oc.System(name='tds')
        mesh = oc.Mesh(p1=(0, 0, 0),
                       p2=(100e-9, 100e-9, 10e-9),
                       cell=(10e-9, 10e-9, 10e-9))
        self.system.hamiltonian += oc.Exchange(1.5e-11)
        self.system.hamiltonian += oc.Demag()
        self.system.dynamics += oc.Precession(2.211e5)
        self.system.dynamics += oc.Damping(0.02)
        self.system.m = df.Field(mesh, value=(0, 1, 0), norm=8e5)



import oommfc as oc
import oommfc.util as ou
from .driver import Driver


class MinDriver(Driver):
    """Minimisation driver.
    
    This class is used for collecting additional parameters, which
    could be passed to `Oxs_MinDriver`. Only parameters which are
    defined in `_allowed_kwargs` can be passed.

    Examples
    --------
    1. Defining driver

    >>> import oommfc as oc
    ...
    >>> md = oc.MinDriver()

    2. Passing an argument which is not allowed

    >>> import oommfc as oc
    ...
    >>> md = oc.MinDriver(myarg=3)
    Traceback (most recent call last):
       ...
    AttributeError: ...

    """
    _allowed_kwargs = ['evolver',
                       'stopping_mxHxm',
                       'stage_iteration_limit',
                       'total_iteration_limit',
                       'stage_count',
                       'stage_count_check',
                       'checkpoint_file',
                       'checkpoint_interval',
                       'checkpoint_disposal',
                       'start_iteration',
                       'start_stage',
                       'start_stage_iteration',
                       'start_stage_start_time',
                       'start_stage_elapsed_time',
                       'start_last_timestep',
                       'normalize_aveM_output',
                       'report_max_spin_angle',
                       'report_wall_time']

    def _script(self, system):
        # Save initial magnetisation.
        m0mif, m0name, Msname = ou.setup_m0(system.m, 'm0')
        mif = m0mif

        # Evolver
        if not hasattr(self, 'evolver'):
            self.evolver = oc.CGEvolver()
        if isinstance(self.evolver, oc.CGEvolver):
            mif += self.evolver._script
        else:
            msg = 'Evolver must be CGEvolver.'
            raise ValueError(msg)

        # Minimisation driver
        mif += '# MinDriver\n'
        mif += 'Specify Oxs_MinDriver {\n'
        mif += '  evolver :evolver\n'
        mif += '  mesh :mesh\n'
        mif += f'  Ms :{Msname}\n'
        mif += f'  m0 :{m0name}\n'
        # Setting stopping mxHxm default value.
        if not hasattr(self, 'stopping_mxHxm'):
            self.stopping_mxHxm = 0.01
        # Other parameters for MinDriver
        for kwarg in self._allowed_kwargs:
            if hasattr(self, kwarg) and kwarg != 'evolver':
                mif += f'  {kwarg} {getattr(self, kwarg)}\n'
        mif += '}\n\n'

        # Saving results
        mif += 'Destination table mmArchive\n'
        mif += 'Destination mags mmArchive\n\n'
        mif += 'Schedule DataTable table Stage 1\n'
        mif += 'Schedule Oxs_MinDriver::Magnetization mags Stage 1'

        return mif

    def _checkargs(self, **kwargs):
        pass



import oommfc.util as ou
import discretisedfield as df


class Mesh(df.Mesh):
    """Finite difference mesh.

    A rectangular mesh domain spans between two points
    :math:`\\mathbf{p}_{1}` and :math:`\\mathbf{p}_{2}`. The domain is
    discretised using a finite difference cell, whose dimensions are
    defined with `cell`. Alternatively, the domain can be discretised
    by passing the number of discretisation cells `n` in all three
    dimensions. Either `cell` or `n` should be defined to discretise
    the domain, not both. Periodic boundary conditions can be
    specified by passing `pbc` argument, which is an iterable
    containing one or more elements from ``['x', 'y', 'z']``. The
    parameter `name` is optional and defaults to 'mesh'.

    In order to properly define a mesh, the length of all mesh domain
    edges must not be zero and the mesh domain must be an aggregate of
    discretisation cells.

    Parameters
    ----------
    p1, p2 : (3,) array_like
        Points between which the mesh domain spans :math:`\\mathbf{p}
        = (p_{x}, p_{y}, p_{z})`.
    cell : (3,) array_like, optional
        Discretisation cell size :math:`(d_{x}, d_{y}, d_{z})`. Either
        `cell` or `n` should be defined, not both.
    n : (3,) array_like, optional
        The number of discretisation cells :math:`(n_{x}, n_{y},
        n_{z})`. Either `cell` or `n` should be defined, not both.
    pbc : iterable, optional
        Periodic boundary conditions in x, y, or z direction. Its value
        is an iterable consisting of one or more characters `x`, `y`,
        or `z`, denoting the direction(s) in which the mesh is periodic.
    regions : dict, optional
        A dictionary defining regions inside the mesh. The keys of the
        dictionary are the region names (str), whereas the values are
        `discretisedfield.Region` objects.
    name : str, optional
        Mesh name (the default is 'mesh'). The mesh name must be a valid
        Python variable name string. More specifically, it must not
        contain spaces, or start with underscore or numeric character.

    Raises
    ------
    ValueError
        If the length of one or more mesh domain edges is zero, or
        mesh domain is not an aggregate of discretisation cells.

    Examples
    --------
    1. Defining a nano-sized thin film mesh by passing `cell` parameter

    >>> import oommfc as oc
    ...
    >>> p1 = (-50e-9, -25e-9, 0)
    >>> p2 = (50e-9, 25e-9, 5e-9)
    >>> cell = (1e-9, 1e-9, 0.1e-9)
    >>> name = 'mesh_name'
    >>> mesh = oc.Mesh(p1=p1, p2=p2, cell=cell, name=name)

    2. Defining a nano-sized thin film mesh by passing `n` parameter

    >>> import oommfc as oc
    ...
    >>> p1 = (-50e-9, -25e-9, 0)
    >>> p2 = (50e-9, 25e-9, 5e-9)
    >>> n = (100, 50, 5)
    >>> name = 'mesh_name'
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, name=name)

    3. Defining a mesh with periodic boundary conditions in x and y
    directions.

    >>> import oommfc as oc
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (100, 100, 1)
    >>> n = (100, 100, 1)
    >>> pbc = 'xy'
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, pbc=pbc, name=name)

    4. Defining a mesh with two regions.

    >>> import oommfc as oc
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (100, 100, 100)
    >>> n = (10, 10, 10)
    >>> regions = {'r1': df.Region(p1=(0, 0, 0), p2=(50, 100, 100)),
    ...            'r2': df.Region(p1=(50, 0, 0), p2=(100, 100, 100))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)

    5. An attempt to define a mesh with invalid parameters, so that
    the ``ValueError`` is raised. In this example, the mesh domain is
    not an aggregate of discretisation cells in the :math:`z`
    direction.

    >>> import oommfc as oc
    ...
    >>> p1 = (-25, 3, 0)
    >>> p2 = (25, 6, 1)
    >>> cell = (5, 3, 0.4)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, cell=cell, name=name)
    Traceback (most recent call last):
        ...
    ValueError: ...

    """
    @property
    def _script(self):
        mif = ''
        if self.regions:
            # The mesh is composed of regions. Multiple BoxAtlas
            # scripts are created and the main MultiAtlas.
            for name, region in self.regions.items():
                mif += ou.mif_box_atlas(region.pmin, region.pmax, name=name)
            mif += '# MultiAtlas\n'
            mif += 'Specify Oxs_MultiAtlas:main_atlas {\n'
            for name in self.regions.keys():
                mif += f'  atlas :{name}_atlas\n'
            mif += f'  xrange {{{self.pmin[0]} {self.pmax[0]}}}\n'
            mif += f'  yrange {{{self.pmin[1]} {self.pmax[1]}}}\n'
            mif += f'  zrange {{{self.pmin[2]} {self.pmax[2]}}}\n'
            mif += '}\n\n'
        else:
            # There are no regions in the mesh.
            mif += ou.mif_box_atlas(self.pmin, self.pmax, name='main')

        if self.pbc:
            mif += '# PeriodicRectangularMesh\n'
            mif += 'Specify Oxs_PeriodicRectangularMesh:mesh {\n'
            mif += '  cellsize {{{} {} {}}}\n'.format(*self.cell)
            mif += '  atlas :main_atlas\n'
            mif += '  periodic {}\n'.format(''.join(sorted(self.pbc)))
            mif += '}\n\n'
        else:
            mif += '# RectangularMesh\n'
            mif += 'Specify Oxs_RectangularMesh:mesh {\n'
            mif += '  cellsize {{{} {} {}}}\n'.format(*self.cell)
            mif += '  atlas :main_atlas\n'
            mif += '}\n\n'

        return mif



import os
import shutil
import oommfc as oc
import discretisedfield as df


def test_multiple_runs():
    name = 'multiple_runs'
    if os.path.exists(name):
        shutil.rmtree(name)

    p1 = (0, 0, 0)
    p2 = (5e-9, 5e-9, 5e-9)
    n = (2, 2, 2)
    Ms = 1e6
    A = 1e-12
    H = (0, 0, 1e6)
    mesh = oc.Mesh(p1=p1, p2=p2, n=n)

    system = oc.System(name=name)
    system.hamiltonian = oc.Exchange(A=A) + oc.Zeeman(H=H)
    system.dynamics = oc.Precession(gamma=oc.consts.gamma0) + oc.Damping(alpha=1)
    system.m = df.Field(mesh, dim=3, value=(0, 0.1, 1), norm=Ms)

    md = oc.MinDriver()
    md.drive(system)
    
    dirname = os.path.join(name, 'drive-0')
    assert os.path.exists(dirname)

    system.hamiltonian.zeeman.H = (1e6, 0, 0)
    
    td = oc.TimeDriver()
    td.drive(system, t=100e-12, n=10)

    dirname = os.path.join(name, 'drive-1')
    assert os.path.exists(dirname)

    system.delete()



import micromagneticmodel as mm


class Dynamics(mm.Dynamics):
    """Dynamics equation

    This class implements the sum of individual dynamics terms. It is
    obtained as a result of addition of two or more dynamics terms.

    Examples
    --------
    1. Adding a dynamics term to the dynamics equation.

    >>> import oommfc as oc
    ...
    >>> dynamics = oc.Dynamics()
    >>> dynamics += oc.Precession(gamma=oc.consts.gamma0)

    2. Creating the Hamiltoninan as a sum of two energy terms

    >>> import oommfc as oc
    ...
    >>> dynamics = oc.Precession(gamma=oc.consts.gamma0) + oc.Damping(alpha=0.1)

    """
    pass



import numbers
import oommfc.util as ou
import discretisedfield as df
import micromagneticmodel as mm


class Exchange(mm.Exchange):
    """Exchange energy term.

    This energy term models ferromagnetic exchange energy term,
    defined by the exchange energy constant `A`.

    .. math::

           w_{ex} = A(\\nabla\\mathbf{m})^{2}

    `A` is a scalar value with J/m units.

    `A` can be either contant in space or spatially varying. If it is
    constant, a single value is passed, e.g. `A = 1e-12`. On the other
    hand, if it varies in space, there are two ways how that can be
    defined. The first one is using a dictionary, where the keys must
    be the same as the names of regions used when the mesh was defined
    (`discretisedfield.Mesh.regions`) and the values are single scalar
    values. In addition, because the exhange energy constant is
    defined between discretisation cells, `A` value must be defined
    between regions as well. For example, to define `A` between
    regions, region names in the dictionary key are separated with
    `':'`. For instance, `A = {'region1': 1e-12, 'region2': 0,
    'region1:region2': 0.5e-12}`. Another way of defining a spatially
    varying parameter is by passing a `discretisedfield.Field`
    object. In this case a single value is associated to every cell
    and the exchange constant between two cells is defined as their
    average. The field must be defined on the same mesh as the
    micromagnetic system and it must have an appropriate dimension
    `dim=1`.

    Parameters
    ----------
    A : number.Real, dict, discretisedfield.Field
      The exchange energy constant
    name: str (optional)
      Name of the energy term object. Defaults to `'exchange'`

    Examples
    --------
    1. Defining spatially constant exchange energy term.

    >>> import oommfc as oc
    ...
    >>> A = 1e-12
    >>> ex = oc.Exchange(A=A)

    2. Defining spatially varying exchange energy consant using a
    dictionary.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> regions = {'region1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 4e-9)),
    ...            'region1': df.Region(p1=(0, 0, 4e-9), p2=(5e-9, 5e-9, 8e-9))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)
    ...
    >>> A = {'region1': 1e-12, 'region2': 0, 'region1:region2': 0.5e-12}
    >>> ex = oc.Exchange(A=A)

    3. Defining spatially varying exchange energy constant using a
    `discretisedfield.Field` object.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n)
    >>> def A_value(pos):
    ...     x, y, z = pos
    ...     if z <= 4e-9:
    ...         return 0
    ...     else:
    ...         return 8.78e-11
    ...
    >>> A = df.Field(mesh, dim=3, value=A_value)
    >>> ex = oc.Exchange(A=A)

    """
    @property
    def _script(self):
        if isinstance(self.A, numbers.Real):
            mif = '# UniformExchange\n'
            mif += 'Specify Oxs_UniformExchange {\n'
            mif += f'  A {self.A}\n'
            mif += '}\n\n'
        elif isinstance(self.A, dict):
            if 'default' in self.A.keys():
                default_value = self.A['default']
            else:
                default_value = 0
            mif = '# Exchange6Ngbr\n'
            mif += 'Specify Oxs_Exchange6Ngbr {\n'
            mif += f'  default_A {default_value}\n'
            mif += '  atlas :main_atlas\n'
            mif += '  A {\n'
            for key, value in self.A.items():
                if key != 'default':
                    if ':' in key:
                        region1, region2 = key.split(':')
                    else:
                        region1, region2 = key, key
                    mif += f'    {region1} {region2} {value}\n'
            mif += '  }\n'
            mif += '}\n\n'
        elif isinstance(self.A, df.Field):
            Amif, Aname = ou.setup_scalar_parameter(self.A, 'ex_A')
            mif = Amif
            mif += '# ExchangePtwise\n'
            mif += 'Specify Oxs_ExchangePtwise {\n'
            mif += f'  A {Aname}\n'
            mif += '}\n\n'

        return mif



import os
import time
import pytest
import pkg_resources
import oommfc.oommf
import oommfc.util
import oommfc.consts
from .hamiltonian import Exchange, UniaxialAnisotropy, \
    Demag, Zeeman, DMI, CubicAnisotropy, Hamiltonian
from .dynamics import Precession, Damping, ZhangLi, Dynamics
from .drivers import MinDriver, TimeDriver
from .evolvers import CGEvolver, EulerEvolver, RungeKuttaEvolver, \
    SpinTEvolver
from .mesh import Mesh
from .system import System
from .data import Data
from . import examples


__version__ = pkg_resources.get_distribution(__name__).version
__dependencies__ = pkg_resources.require(__name__)


def _run_tests(tag):
    args = ['-m', tag, '-v', '--pyargs', 'oommfc']  # pragma: no cover
    return pytest.main(args)  # pragma: no cover


def test():
    return(_run_tests('not travis and not docker'))  # pragma: no cover


def test_not_oommf():
    return(_run_tests('not oommf and not travis'))  # pragma: no cover


def test_oommf():
    return(_run_tests('oommf and not travis '
                      'and not docker'))  # pragma: no cover


def test_docker():
    return(_run_tests('docker'))  # pragma: no cover



import os
import sys
import time
import datetime
import logging
import shutil
import oommfc as oc
import subprocess as sp

log = logging.getLogger(__name__)
_cached_oommf_runner = None


class OOMMFRunner:
    """Base class for running OOMMF.

    Don't use this directly. Use get_oommf_runner() to pick a subclass
    of this class.

    """
    def call(self, argstr, need_stderr=False):
        now = datetime.datetime.now()
        timestamp = '{}/{:02d}/{:02d} {:02d}:{:02d}'.format(now.year,
                                                            now.month,
                                                            now.day,
                                                            now.hour,
                                                            now.minute)
        print('{}: Running OOMMF ({}) ... '.format(timestamp, argstr), end='')

        tic = time.time()
        res = self._call(argstr=argstr, need_stderr=need_stderr)
        self._kill()
        toc = time.time()
        seconds = '({:0.1f} s)'.format(toc - tic)
        print(seconds)

        if res.returncode is not 0:
            if sys.platform != 'win32':
                # Only on Linux and MacOS - on Windows we do not get
                # stderr and stdout.
                stderr = res.stderr.decode('utf-8', 'replace')
                stdout = res.stdout.decode('utf-8', 'replace')
                cmdstr = ' '.join(res.args)
                print('OOMMF error:')
                print('\tcommand: {}'.format(cmdstr))
                print('\tstdout: {}'.format(stdout))
                print('\tstderr: {}'.format(stderr))
                print('\n')
            raise RuntimeError('Error in OOMMF run.')

        return res

    def _call(self, argstr, need_stderr=False):
        # This method should be implemented in subclass.
        raise NotImplementedError

    def _kill(self, targets=('all',)):
        # This method should be implemented in subclass.
        raise NotImplementedError
    
    def version(self):
        res = self.call(argstr='+version', need_stderr=True)
        return res.stderr.decode('utf-8').split('oommf.tcl')[-1].strip()

    def platform(self):
        res = self.call(argstr='+platform', need_stderr=True)
        return res.stderr.decode('utf-8')

    
class TclOOMMFRunner(OOMMFRunner):
    """Using path to oommf.tcl.

    """
    def __init__(self, oommf_tcl):
        self.oommf_tcl = oommf_tcl  # a path to oommf.tcl

    def _call(self, argstr, need_stderr=False):
        cmd = ['tclsh', self.oommf_tcl, 'boxsi', '+fg',
               argstr, '-exitondone', '1']

        # Not clear why we cannot get stderr and stdout on
        # win32. Calls to OOMMF get stuck.
        stdout = stderr = sp.PIPE
        if sys.platform == 'win32' and not need_stderr:
            stdout = stderr = None

        return sp.run(cmd, stdout=stdout, stderr=stderr)

    def _kill(self, targets=['all']):
        sp.run(["tclsh", self.oommf_tcl, "killoommf"] + targets)


class ExeOOMMFRunner(OOMMFRunner):
    """Using oommf executable on $PATH.

    """
    def __init__(self, oommf_exe='oommf'):
        self.oommf_exe = oommf_exe

    def _call(self, argstr, need_stderr=False):
        # Here we might need stderr = stdot = None like in
        # TclOOMMFRunner for Windows.  This is not clear because we
        # never use ExeOOMMFRunner on Windows.

        cmd = [self.oommf_exe, 'boxsi', '+fg', argstr, '-exitondone', '1']
        return sp.run(cmd, stdout=sp.PIPE, stderr=sp.PIPE)

    def _kill(self, targets=['all']):
        sp.run([self.oommf_exe, "killoommf"] + targets)


class DockerOOMMFRunner(OOMMFRunner):
    """Run OOMMF in a docker container.

    """
    def __init__(self, docker_exe='docker', image='joommf/oommf'):
        self.image = image
        self.docker_exe = docker_exe

    def _call(self, argstr, need_stderr=False):
        cmd = [self.docker_exe, 'run', '-v', os.getcwd()+':/io',
               self.image, '/bin/bash', '-c',
               ('tclsh /usr/local/oommf/oommf/oommf.tcl boxsi '
                '+fg {} -exitondone 1').format(argstr)]
        return sp.run(cmd, stdout=sp.PIPE, stderr=sp.PIPE)

    def _kill(self, targets=('all',)):
        # There is no need to kill OOMMF when run inside docker.
        pass


def get_oommf_runner(use_cache=True, envvar='OOMMFTCL',
                     oommf_exe='oommf', docker_exe='docker'):
    """Find the best available way to run OOMMF.

    Returns an OOMMFRunner object, or raises EnvironmentError if no suitable
    method is found.

    Parameters
    ----------
    use_cache : bool
      The first call to this function will determine the best way to run OOMMF
      and cache it. Normally, subsequent calls will return the OOMMFRunner
      object from the cache. Setting this parameter to False will cause it to
      check for available methods again.
    envvar : str
      Name of the environment variable containing the path to oommf.tcl
    oommf_exe : str
      The name or path of the executable oommf command
    docker_exe : str
      The name or path of the docker command

    """
    global _cached_oommf_runner
    if use_cache and (_cached_oommf_runner is not None):
        return _cached_oommf_runner

    # Check for OOMMFTCL environment variable pointing to oommf.tcl.
    oommf_tcl = os.environ.get(envvar, None)
    if oommf_tcl is not None:
        cmd = ['tclsh', oommf_tcl, 'boxsi',
               '+fg', '+version', '-exitondone', '1']
        try:
            res = sp.run(cmd, stdout=sp.PIPE, stderr=sp.PIPE)
        except FileNotFoundError:
            log.warning('oommf.tcl was not found.')
        else:
            if res.returncode:
                log.warning('OOMMFTCL is set, but OOMMF could not be run.\n'
                            'stdout:\n{}\n'
                            'stderr:\n{}'.format(res.stdout, res.stderr))
            else:
                _cached_oommf_runner = TclOOMMFRunner(oommf_tcl)
                return _cached_oommf_runner

    # OOMMF is installed via conda and oommf.tcl is in opt/oommf
    # (Windows). This would probably also work on MacOS/Linux, but on
    # these operating systems, when installed via conda, we use
    # 'oommf' executable.
    if sys.platform == 'win32' and \
       os.path.isdir(os.path.join(sys.prefix, 'conda-meta')):
        oommf_tcl = os.path.join(sys.prefix, 'opt', 'oommf', 'oommf.tcl')
        if os.path.isfile(oommf_tcl):
            _cached_oommf_runner = TclOOMMFRunner(oommf_tcl)
            return _cached_oommf_runner

    # OOMMF available as an executable - in a conda env on Mac/Linux,
    # or oommf installed separately.
    oommf_exe = shutil.which(oommf_exe)
    if oommf_exe:
        _cached_oommf_runner = ExeOOMMFRunner(oommf_exe)
        return _cached_oommf_runner

    # Check for docker to run OOMMF in a docker image.
    cmd = [docker_exe, 'images']
    try:
        res = sp.run(cmd, stdout=sp.PIPE, stderr=sp.PIPE)
    except FileNotFoundError:
        log.warning('Docker was not found.')
    else:
        if res.returncode:
            log.warning('Error running docker\n'
                        'stdout:\n{}\n'
                        'stderr:\n{}'.format(res.stdout, res.stderr))
        else:
            _cached_oommf_runner = DockerOOMMFRunner(docker_exe=docker_exe,
                                                     image='joommf/oommf')
            return _cached_oommf_runner

    # If OOMMFRunner was not returned up to this point, we raise an
    # exception.
    raise EnvironmentError('Cannot find OOMMF.')


def status():
    """Run a macrospin example for 1 ps through oommfc and print the OOMMF
    status.

    """
    try:
        system = oc.examples.macrospin()
        td = oc.TimeDriver()
        td.drive(system, t=1e-12, n=1, overwrite=True)
        print('OOMMF found and running.')
        shutil.rmtree('example-macrospin')
        return 0
    except (EnvironmentError, RuntimeError):
        print("Cannot find OOMMF.")
        return 1

def overhead():
    """Run a macrospin example for 1 ps through oommfc and directly and
    return the difference in run times.

    Returns
    -------
    overhead : float
      The time difference (overhead) between running OOMMF though
      oommfc and directly

    """
    # Running OOMMF through oommfc.
    system = oc.examples.macrospin()
    td = oc.TimeDriver()
    oommfc_start = time.time()
    td.drive(system, t=1e-12, n=1, overwrite=True)
    oommfc_stop = time.time()
    oommfc_time = oommfc_stop - oommfc_start

    # Running OOMMF directly.
    oommf_runner = get_oommf_runner()
    mifpath = os.path.realpath(os.path.join('example-macrospin', 'drive-0',
                                            'example-macrospin.mif'))
    oommf_start = time.time()
    oommf_runner.call(mifpath)
    oommf_stop = time.time()
    oommf_time = oommf_stop - oommf_start
    shutil.rmtree('example-macrospin')

    return oommfc_time - oommf_time



import os
import shutil
import pytest
import oommfc as oc
import discretisedfield as df


class TestDerive:
    def setup(self):
        self.name = "derive_tests"
        if os.path.exists(self.name):
            shutil.rmtree(self.name)

        self.system = oc.System(name=self.name)
        self.system.hamiltonian += oc.Exchange(A=1e-12)
        self.system.hamiltonian += oc.Demag()
        self.system.hamiltonian += oc.Zeeman(H=(8e6, 0, 0))
        self.system.hamiltonian += oc.UniaxialAnisotropy(K1=1e4, u=(0, 0, 1))
        mesh = oc.Mesh(p1=(0, 0, 0), p2=(4e-9, 4e-9, 2e-9),
                       cell=(1e-9, 1e-9, 1e-9))
        self.system.m = df.Field(mesh, value=(0, 0, 1), norm=8e6)

    def test_energy(self):
        hamiltonian = self.system.hamiltonian
        assert isinstance(hamiltonian.exchange.energy, float)
        assert isinstance(hamiltonian.demag.energy, float)
        assert isinstance(hamiltonian.zeeman.energy, float)
        assert isinstance(hamiltonian.uniaxialanisotropy.energy, float)
        assert isinstance(hamiltonian.energy, float)
        if os.path.exists(self.name):
            shutil.rmtree(self.name)

    def test_energy_density(self):
        hamiltonian = self.system.hamiltonian
        assert isinstance(hamiltonian.exchange.energy_density, df.Field)
        assert isinstance(hamiltonian.demag.energy_density, df.Field)
        assert isinstance(hamiltonian.zeeman.energy_density, df.Field)
        assert isinstance(hamiltonian.uniaxialanisotropy.energy_density,
                          df.Field)
        assert isinstance(hamiltonian.energy_density, df.Field)
        assert hamiltonian.energy_density.dim == 1
        if os.path.exists(self.name):
            shutil.rmtree(self.name)

    def test_effective_field(self):
        hamiltonian = self.system.hamiltonian
        assert isinstance(hamiltonian.exchange.effective_field, df.Field)
        assert isinstance(hamiltonian.demag.effective_field, df.Field)
        assert isinstance(hamiltonian.zeeman.effective_field, df.Field)
        assert isinstance(hamiltonian.uniaxialanisotropy.effective_field,
                          df.Field)
        assert isinstance(hamiltonian.effective_field, df.Field)
        assert hamiltonian.effective_field.dim == 3
        if os.path.exists(self.name):
            shutil.rmtree(self.name)



import micromagneticmodel as mm


class ZhangLi(mm.ZhangLi):
    """Zhang-Li spin-transfer torque dynamics term.

    This dynamics term models the Zhang-Li spin-transfer-torque term
    in the Landau-Lifshitz-Gilbert equation. It is defined by the
    velocity in the x-direction `u` and the non-adiabatic constant
    `beta`.

    .. math::

           \\frac{\\text{d}\\mathbf{m}}{\\text{d}t} = -(\\mathbf{u}
           \\cdot \\boldsymbol\\nabla)\\mathbf{m} + \\beta\\mathbf{m}
           \\times \\big[(\\mathbf{u} \\cdot
           \\boldsymbol\\nabla)\\mathbf{m}\\big]

    `u` can be either contant in space or spatially varying, whereas
    `beta` can be only constant in space. If `u` is constant, a single
    value is passed, e.g. `u = 0.1`. On the other hand, if it varies
    in space, there are two ways how that can be defined. The first
    one is using a dictionary, where the keys must be the same as the
    names of regions used when the mesh was defined
    (`discretisedfield.Mesh.regions`) and the values are single scalar
    values. For instance, `u = {'region1': 0.1, 'region2': 0.2}.
    Another way of defining a spatially varying parameter is by
    passing a `discretisedfield.Field` object. The field must be
    defined on the same mesh as the micromagnetic system and it must
    have an appropriate dimension `dim=1`.

    Parameters
    ----------
    u : number.Real, dict, discretisedfield.Field
      Velocity in the x-direction.
    beta : numbers.Real
      Non-adiabatic factor
    name: str (optional)
      Name of the dynamics term object. Defaults to `'zhangli'`

    Examples
    --------
    1. Defining spatially constant Zhang-Li term.

    >>> import oommfc as oc
    ...
    >>> u = 0.1
    >>> beta = 0.5
    >>> zl = oc.ZhangLi(u=u, beta=beta)
    >>> zl.u
    0.1

    2. Defining spatially varying velocity using a dictionary.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> regions = {'region1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 4e-9)),
    ...            'region1': df.Region(p1=(0, 0, 4e-9), p2=(5e-9, 5e-9, 8e-9))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)
    ...
    >>> u = {'region1': 0.1, 'region2': 0.01}
    >>> beta = 0.5
    >>> zl = oc.ZhangLi(u=u, beta=beta)

    3. Defining spatially varying velocity using a
    `discretisedfield.Field` object.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n)
    >>> def u_value(pos):
    ...     x, y, z = pos
    ...     if z <= 4e-9:
    ...         return 0.1
    ...     else:
    ...         return 0.2
    ...
    >>> u = df.Field(mesh, dim=3, value=u_value)
    >>> zl = oc.ZhangLi(u=u, beta=beta)

    """
    pass



import sys
import numbers
import micromagneticmodel as mm


class DMI(mm.DMI):
    """Dzyaloshinskii-Moriya energy term.

    This energy term models DM energy term, defined by the DM energy
    constant `D` and crystalographic class `crystalclass`. `D` is a
    scalar value with J/m2 units.

    .. math::

           \\mathbf{w_\\text{dmi}} = \\left\\{
           \\begin{array}{ll}
           D \\mathbf{m} \\cdot (\\nabla \\times \\mathbf{m}), &
           \\text{for}\\,\\,T(O) \\\\ D ( \\mathbf{m} \\cdot \\nabla m_{z} -
           m_{z} \\nabla \\cdot \\mathbf{m}), & \\text{for}\\,\\,C_{nv} \\\\
           D\\mathbf{m} \\cdot \\left( \\frac{\\partial
           \\mathbf{m}}{\\partial x} \\times \\hat{x} - \\frac{\\partial
           \\mathbf{m}}{\partial y} \\times \\hat{y} \\right), &
           \\text{for}\\,\\,D_{2d} \\\\ 
           \\end{array} \\right.

    `D` can be either contant in space or spatially varying. If it is
    constant, a single value is passed, e.g. `D = 1e-3`. On the other
    hand, if it varies in space, there is only one way how that can be
    defined: using a dictionary. The keys must be the same as the
    names of regions used when the mesh was defined
    (`discretisedfield.Mesh.regions`) and the values are single scalar
    values. In addition, because the DM energy constant is defined
    between discretisation cells, `D` value must be defined between
    regions as well. For example, to define `D` between regions,
    region names in the dictionary key are separated with `':'`. For
    instance, `D = {'region1': 1e-3, 'region2': 0, 'region1:region2':
    0.5e-3}`. This term cannot be set using `discretisedfield.Field`
    object.

    Parameters
    ----------
    D : number.Real, dict
      The Dzyaloshinskii-Moriya energy constant
    name: str (optional)
      Name of the energy term object. Defaults to `'dmi'`

    Examples
    --------
    1. Defining spatially constant DM energy term.

    >>> import oommfc as oc
    ...
    >>> D = 1e-3
    >>> dmi = oc.DMI(D=D, crystalclass='Cnv')

    2. Defining spatially varying DM energy consant using a
    dictionary.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> regions = {'region1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 4e-9)),
    ...            'region1': df.Region(p1=(0, 0, 4e-9), p2=(5e-9, 5e-9, 8e-9))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)
    ...
    >>> D = {'region1': 1e-3, 'region2': 0, 'region1:region2': 0.5e-3}
    >>> dmi = oc.DMI(D=D, crystalclass='D2d')

    """
    @property
    def _script(self):
        if self.crystalclass in ['t', 'o'] and sys.platform != 'win32':
            oxs = 'Oxs_DMI_T'
        elif self.crystalclass == 'd2d' and sys.platform != 'win32':
            oxs = 'Oxs_DMI_D2d'
        elif self.crystalclass == 'cnv' and sys.platform != 'win32':
            oxs = 'Oxs_DMI_Cnv'
        elif self.crystalclass == 'cnv' and sys.platform == 'win32':
            oxs = 'Oxs_DMExchange6Ngbr'
        else:
            raise ValueError(f'The {self.crystalclass} crystal class is not '
                             f'supported on {sys.platform} platform.')

        mif = f'# DMI of crystallographic class {self.crystalclass}\n'
        mif += f'Specify {oxs} {{\n'
        if isinstance(self.D, numbers.Real):
            mif += f'  default_D {self.D}\n'
            mif += '  atlas :main_atlas\n'
            mif += '  D {\n'
            mif += f'    main main {self.D}\n'
            mif += '  }\n'
            mif += '}\n\n'
        elif isinstance(self.D, dict):
            if 'default' in self.D.keys():
                default_value = self.D['default']
            else:
                default_value = 0
            mif += f'  default_D {default_value}\n'
            mif += '  atlas :main_atlas\n'
            mif += '  D {\n'
            for key, value in self.D.items():
                if key != 'default':
                    if ':' in key:
                        region1, region2 = key.split(':')
                    else:
                        region1, region2 = key, key
                    mif += f'    {region1} {region2} {value}\n'
            mif += '  }\n'
            mif += '}\n\n'
        else:
            msg = f'Type {type(self.D)} not supported.'
            raise ValueError(msg)

        return mif



import oommfc.util as ou
import micromagneticmodel as mm


class UniaxialAnisotropy(mm.UniaxialAnisotropy):
    """Uniaxial anisotropy energy term.

    This energy term models uniaxial anisotropy term, defined by the
    anisotropy constant `K1` and the uniaxial anisotropy axis
    `u`.

    .. math::

           w_{ua} = -K_{1}(\\mathbf{m} \\cdot \\mathbf{u})^{2}

    `K1` is a scalar, given in J/m3, whereas `u` is a three-dimensional
    vector with real component values and no units. The axis direction
    must be non-zero at each point in the mesh, and will be normalized
    to unit magnitude before being used.

    Both the scalar parameter `K1` and the vector parameter `u` can be
    either contant is space or spatially varying. If they are
    constant, a single value is passed, e.g. `K1 = 1e5` and `u = (0,
    0, 1)`. On the other hand, if one of them varies in space, there
    are two ways how they can be defined. The first one is using
    dictionaries, where the keys must be the same as the names of
    regions used when the mesh was defined
    (`discretisedfield.Mesh.regions`) and the values are single scalar
    or vector values. For instance, `K1 = {'region1': 1e5, 'region2':
    0}` and `u = {'region1': (0, 0, 1), 'region2': (1, 0,
    0)}`. Another way of defining a spatially varying parameter is by
    passing a `discretisedfield.Field` object. The field must be
    defined on the same mesh as the micromagnetic system and it must
    have an appropriate dimension (`dim=1` for `K1` and `dim=3` for
    `u`).

    Parameters
    ----------
    K1 : numbers.Real, dict, discretisedfield.Field
      The uniaxial anisotropy constant
    u : array_like, dict, discretisedfield.Field
      The uniaxial anisotropy axis
    name: str (optional)
      Name of the energy term object. Defaults to `'uniaxialanisotropy'`

    Examples
    --------
    1. Defining spatially constant uniaxial anisotropy

    >>> import oommfc as oc
    ...
    >>> K1 = 1e5
    >>> u = (0, 0, 1)
    >>> ua = oc.UniaxialAnisotropy(K1=K1, u=u)

    2. Defining spatially varying uniaxial anisotropy using a
    dictionary

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> regions = {'region1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 4e-9)),
    ...            'region1': df.Region(p1=(0, 0, 4e-9), p2=(5e-9, 5e-9, 8e-9))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)
    ...
    >>> K1 = {'region1': 1e5, 'region2': 1e3}
    >>> u = (0, 0, 1)
    >>> ua = oc.UniaxialAnisotropy(K1=K1, u=u)

    3. Defining spatially varying uniaxial anisotropy using a
    `discretisedfield.Field` object.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n)
    >>> def u_value(pos):
    ...     x, y, z = pos
    ...     if z <= 4e-9:
    ...         return (1, 0, 0)
    ...     else:
    ...         return (0, 0, 1)
    ...
    >>> K1 = 1e5
    >>> u = df.Field(mesh, dim=3, value=u_value)
    >>> ua = oc.UniaxialAnisotropy(K1=K1, u=u)

    """
    @property
    def _script(self):
        k1mif, k1name = ou.setup_scalar_parameter(self.K1, 'ua_K1')
        umif, uname = ou.setup_vector_parameter(self.u, 'ua_u')

        mif = ''
        mif += k1mif
        mif += umif
        mif += '# UniaxialAnisotropy\n'
        mif += 'Specify Oxs_UniaxialAnisotropy {\n'
        mif += f'  K1 {k1name}\n'
        mif += f'  axis {uname}\n'
        mif += '}\n\n'

        return mif



import os
import shutil
import numpy as np
import oommfc as oc
import discretisedfield as df


class TestZeeman:
    def setup(self):
        self.p1 = (-10e-9, -5e-9, -3e-9)
        self.p2 = (10e-9, 5e-9, 3e-9)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.regions = {'r1': df.Region(p1=(-10e-9, -5e-9, -3e-9), p2=(10e-9, 0, 3e-9)),
                        'r2': df.Region(p1=(-10e-9, 0, -3e-9), p2=(10e-9, 5e-9, 3e-9))}

    def test_vector(self):
        name = 'ze_vector'
        if os.path.exists(name):
            shutil.rmtree(name)

        H = (0, 0, 1e6)
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell)

        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(0, 1, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        system.delete()

    def test_field(self):
        name = 'ze_field'
        if os.path.exists(name):
            shutil.rmtree(name)

        def value_fun(pos):
            x, y, z = pos
            if x <= 0:
                return (1e6, 0, 0)
            else:
                return (0, 0, 1e6)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell)
            
        H = df.Field(mesh, dim=3, value=value_fun)
        Ms = 1e6

        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(0, 1, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m((-2e-9, -2e-9, -2e-9))
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        value = system.m((2e-9, 2e-9, 2e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        system.delete()

    def test_dict(self):
        name = 'ze_dict'
        if os.path.exists(name):
            shutil.rmtree(name)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell,
                       regions=self.regions)
            
        H = {'r1': (1, 0, 0), 'r2': (0, 0, 1)}
        Ms = 1e6

        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(0, 1, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m((-2e-9, -2e-9, -2e-9))
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        value = system.m((2e-9, 2e-9, 2e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        system.delete()



import os
import glob
import shutil
import pytest
import oommfc as oc
import numpy as np
import discretisedfield as df


class TestTimeDriver:
    def setup(self):
        p1 = (0, 0, 0)
        p2 = (5e-9, 5e-9, 5e-9)
        n = (2, 2, 2)
        self.Ms = 1e6
        A = 1e-12
        H = (0, 0, 1e6)
        self.mesh = oc.Mesh(p1=p1, p2=p2, n=n)
        self.hamiltonian = oc.Exchange(A=A) + oc.Zeeman(H=H)
        self.precession = oc.Precession(gamma=oc.consts.gamma0)
        self.damping = oc.Damping(alpha=1)
        self.m = df.Field(self.mesh, dim=3, value=(0, 0.1, 1), norm=self.Ms)
    
    def test_noevolver_nodriver(self):
        name = 'td_noevolver_nodriver'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = oc.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        system.delete()

    def test_rungekutta_evolver_nodriver(self):
        name = 'td_rungekutta_evolver_nodriver'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.precession + self.damping
        system.m = self.m

        evolver = oc.RungeKuttaEvolver(method='rkf54s')
        td = oc.TimeDriver(evolver=evolver)
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        system.delete()

    def test_euler_evolver_nodriver(self):
        name = 'td_euler_evolver_nodriver'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.precession + self.damping
        system.m = self.m

        evolver = oc.EulerEvolver(start_dm=0.02)
        td = oc.TimeDriver(evolver=evolver)
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        system.delete()

    def test_noevolver_driver(self):
        name = 'td_noevolver_driver'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = oc.TimeDriver(stopping_dm_dt=0.01)
        td.drive(system, t=0.3e-9, n=50)

        value = system.m(self.mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        system.delete()

    def test_noprecession(self):
        name = 'td_noprecession'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.damping
        system.m = self.m

        td = oc.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        system.delete()

    def test_nodamping(self):
        name = 'td_nodamping'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.precession
        system.m = self.m

        td = oc.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) > 1e3

        system.delete()
        
    def test_output_files(self):
        name = 'td_output_files'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = oc.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        dirname = os.path.join(f'{name}', f'drive-{system.drive_number-1}')
        assert os.path.exists(dirname)
        miffilename = os.path.join(dirname, f'{name}.mif')
        assert os.path.isfile(miffilename)
        omf_files = list(glob.iglob(os.path.join(dirname, '*.omf')))
        assert len(omf_files) == 51
        odt_files = list(glob.iglob(os.path.join(dirname, '*.odt')))
        assert len(odt_files) == 1
        omffilename = os.path.join(dirname, 'm0.omf')
        assert omffilename in omf_files

        system.delete()

    def test_drive_exception(self):
        name = 'td_exception'
        if os.path.exists(name):
            shutil.rmtree(name)

        system = oc.System(name=name)
        system.hamiltonian = self.hamiltonian
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = oc.TimeDriver()
        with pytest.raises(ValueError):
            td.drive(system, t=-0.1e-9, n=10)
        with pytest.raises(ValueError):
            td.drive(system, t=0.1e-9, n=-10)

        system.delete()



import os
import shutil
import numpy as np
import oommfc as oc
import discretisedfield as df


class TestSystem:
    def setup(self):
        self.p1 = (-5e-9, -5e-9, -3e-9)
        self.p2 = (5e-9, 5e-9, 3e-9)
        self.n = (10, 10, 10)

    def test_system(self):
        name = 'test_system'
        if os.path.exists(name):
            shutil.rmtree(name)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, n=self.n)

        A = 1e-12
        H = (0, 0, 1e7)
        Ms = 1e6
        
        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H) + oc.Exchange(A=A)
        system.dynamics = oc.Damping(alpha=1)
        system.m = df.Field(mesh, dim=3, value=(0, 0.1, 1), norm=Ms)
        
        td = oc.TimeDriver()
        td.drive(system, t=0.5e-9, n=50)

        # u is zero, nothing should change.
        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        system.delete()



import micromagneticmodel as mm


class Demag(mm.Demag):
    """Demagnetisation energy term.

    This object models micromagnetic demagnetisation energy term. It
    does not take any mandatory arguments. However,
    `asymptotic_radius` and `name` can be passed.

    Parameters
    ----------
    asymptotic_radius : number.Real
      The asymptotic radius
    name: str (optional)
      Name of the energy term object. Defaults to `'demag'`

    Examples
    --------
    1. Initialising the demagnetisation energy term.

    >>> import oommfc as oc
    ...
    >>> demag = oc.Demag()

    """
    @property
    def _script(self):
        mif = '# Demag\n'
        mif += 'Specify Oxs_Demag {\n'
        if hasattr(self, 'asymptotic_radius'):
            mif += f'  asymptotic_radius {self.asymptotic_radius}\n'
        mif += '}\n\n'

        return mif



import oommfc.util as ou
import micromagneticmodel as mm


class Zeeman(mm.Zeeman):
    """Zeeman energy term.

    This energy term models Zeeman energy term, defined by the
    external magnetic field `H`.

    .. math::

           w_{z} = -\\mu_{0}M_\\text{s}\\mathbf{m} \\cdot \\mathbf{H}

    `H` is a three-dimensional vector with real component values and
    A/m units.

    `H` can be either contant in space or spatially varying. If it is
    constant, a single value is passed, e.g. `H = (0, 0, 1e5)`. On the
    other hand, if it varies in space, there are two ways how that can
    be defined. The first one is using a dictionary, where the keys
    must be the same as the names of regions used when the mesh was
    defined (`discretisedfield.Mesh.regions`) and the values are
    single vector values. For instance, `H = {'region1': (0, 0, 1e5),
    'region2': (1e6, 0, 0)}`. Another way of defining a spatially
    varying parameter is by passing a `discretisedfield.Field`
    object. The field must be defined on the same mesh as the
    micromagnetic system and it must have an appropriate dimension
    `dim=3`.

    Parameters
    ----------
    H : array_like, dict, discretisedfield.Field
      The external magnetic field
    name: str (optional)
      Name of the energy term object. Defaults to `'zeeman'`

    Examples
    --------
    1. Defining spatially constant Zeeman energy term.

    >>> import oommfc as oc
    ...
    >>> H = (0, 0, 1e5)
    >>> ze = oc.Zeeman(H=H)

    2. Defining spatially varying external magnetic field.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> regions = {'region1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 4e-9)),
    ...            'region1': df.Region(p1=(0, 0, 4e-9), p2=(5e-9, 5e-9, 8e-9))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)
    ...
    >>> H = {'region1': (1e5, 0, 0), 'region2': (0, 0, 1e5)}
    >>> ze = oc.Zeeman(H=H)

    3. Defining spatially varying external magnetic field using a
    `discretisedfield.Field` object.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n)
    >>> def H_value(pos):
    ...     x, y, z = pos
    ...     if z <= 4e-9:
    ...         return (1e5, 0, 0)
    ...     else:
    ...         return (0, 0, 1e5)
    ...
    >>> H = df.Field(mesh, dim=3, value=H_value)
    >>> ze = oc.Zeeman(H=H)

    """
    @property
    def _script(self):
        Hmif, Hname = ou.setup_vector_parameter(self.H, 'ze_H')

        mif = ''
        mif += Hmif
        mif += '# FixedZeeman\n'
        mif += 'Specify Oxs_FixedZeeman {\n'
        mif += f'  field {Hname}\n'
        mif += '}\n\n'

        return mif



import os
import shutil
import random
import numpy as np
import oommfc as oc
import discretisedfield as df


class TestExchange:
    def setup(self):
        self.p1 = (-5e-9, -5e-9, -3e-9)
        self.p2 = (5e-9, 5e-9, 3e-9)
        self.n = (10, 10, 10)
        self.regions = {'r1': df.Region(p1=(-5e-9, -5e-9, -3e-9), p2=(5e-9, 0, 3e-9)),
                        'r2': df.Region(p1=(-5e-9, 0, -3e-9), p2=(5e-9, 5e-9, 3e-9))}

    def test_scalar(self):
        name = 'ex_scalar'
        if os.path.exists(name):
            shutil.rmtree(name)

        A = 1e-12
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, n=self.n)

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A)

        def m_value(pos):
            return [2*random.random()-1 for i in range(3)]

        system.m = df.Field(mesh, dim=3, value=m_value, norm=Ms)
        
        md = oc.MinDriver()
        md.drive(system)

        assert abs(np.linalg.norm(system.m.average) - Ms) < 1e-3

        system.delete()

    def test_dict(self):
        name = 'ex_dict'
        if os.path.exists(name):
            shutil.rmtree(name)

        A = {'r1': 0, 'r2': 1e-12, 'r1:r2': 1e-12}
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, n=self.n,
                       regions=self.regions)

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A)

        def m_value(pos):
            return [2*random.random()-1 for i in range(3)]

        system.m = df.Field(mesh, dim=3, value=m_value, norm=Ms)
        
        md = oc.MinDriver()
        md.drive(system)

        # A=0 region
        value1 = system.m((1e-9, -4e-9, 3e-9))
        value2 = system.m((1e-9, -2e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value1, value2)) > 1

        # A!=0 region
        value1 = system.m((1e-9, 4e-9, 3e-9))
        value2 = system.m((1e-9, 2e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value1, value2)) < 1

        system.delete()

    def test_field(self):
        name = 'ex_field'
        if os.path.exists(name):
            shutil.rmtree(name)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, n=self.n)

        def A_fun(pos):
            x, y, z = pos
            if x <= 0:
                return 1e-10  # for 0, OOMMF gives nan
            else:
                return 1e-12

        A = df.Field(mesh, dim=1, value=A_fun)
        Ms = 1e6

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A=A)

        def m_value(pos):
            return [2*random.random()-1 for i in range(3)]

        system.m = df.Field(mesh, dim=3, value=m_value, norm=Ms)
        
        md = oc.MinDriver()
        md.drive(system)

        assert abs(np.linalg.norm(system.m.average) - Ms) < 1e-3

        system.delete()



import oommfc as oc
import oommfc.util as ou
from .driver import Driver


class TimeDriver(Driver):
    """Time driver.
    
    This class is used for collecting additional parameters, which
    could be passed to `Oxs_TimeDriver`. Only parameters which are
    defined in `_allowed_kwargs` can be passed.

    Examples
    --------
    1. Defining driver

    >>> import oommfc as oc
    ...
    >>> td = oc.TimeDriver()

    2. Passing an argument which is not allowed

    >>> import oommfc as oc
    ...
    >>> td = oc.TimeDriver(myarg=3)
    Traceback (most recent call last):
       ...
    AttributeError: ...

    """
    _allowed_kwargs = ['evolver',
                       'stopping_dm_dt',
                       'stage_iteration_limit',
                       'total_iteration_limit',
                       'stage_count_check',
                       'checkpoint_file',
                       'checkpoint_interval',
                       'checkpoint_disposal',
                       'start_iteration',
                       'start_stage',
                       'start_stage_iteration',
                       'start_stage_start_time',
                       'start_stage_elapsed_time',
                       'start_last_timestep',
                       'normalize_aveM_output',
                       'report_max_spin_angle',
                       'report_wall_time']

    def _script(self, system, **kwargs):
        # Save initial magnetisation.
        m0mif, m0name, Msname = ou.setup_m0(system.m, 'm0')
        mif = m0mif

        # Extract dynamics equation parameters.
        gamma, alpha, u, beta = None, None, None, None
        for term in system.dynamics.terms:
            if isinstance(term, oc.Precession):
                gamma = term.gamma
            if isinstance(term, oc.Damping):
                alpha = term.alpha
            if isinstance(term, oc.ZhangLi):
                u = term.u
                beta = term.beta

        # Evolver
        if not hasattr(self, 'evolver'):
            if u is not None:
                self.evolver = oc.SpinTEvolver()
            else:
                self.evolver = oc.RungeKuttaEvolver()

        if gamma is not None:
            self.evolver.gamma_G = gamma
        else:
            self.evolver.do_precess = 0  # do_precess default value is 1

        if alpha is not None:
            self.evolver.alpha = alpha
        else:
            self.evolver.alpha = 0

        if u is not None:
            self.evolver.u = u
        if beta is not None:
            self.evolver.beta = beta

        # Evolver script
        if isinstance(self.evolver, (oc.EulerEvolver, oc.RungeKuttaEvolver,
                                     oc.SpinTEvolver)):
            mif += self.evolver._script
        else:
            msg = ('Evolver must be either EulerEvolver, '
                   'RungeKuttaEvolver, or SpinTEvolver')
            raise ValueError(msg)

        # For deriving, a small timestep is chosen.
        if 'derive' in kwargs:
            t, n = 1e-25, 1
            self.total_iteration_limit = 1
        else:
            t, n = kwargs['t'], kwargs['n']

        # TimeDriver
        mif += '# TimeDriver\n'
        mif += 'Specify Oxs_TimeDriver {\n'
        mif += '  evolver :evolver\n'
        mif += '  mesh :mesh\n'
        mif += f'  Ms :{Msname}\n'
        mif += f'  m0 :{m0name}\n'
        mif += f'  stopping_time {t/n}\n'
        mif += f'  stage_count {n}\n'
        # Other parameters for TimeDriver
        for kwarg in self._allowed_kwargs:
            if hasattr(self, kwarg) and kwarg != 'evolver':
                mif += f'  {kwarg} {getattr(self, kwarg)}\n'
        mif += '}\n\n'

        # Saving results
        mif += 'Destination table mmArchive\n'
        mif += 'Destination mags mmArchive\n'
        mif += 'Destination archive mmArchive\n\n'

        if 'derive' in kwargs:
            if 'ield' in kwargs['derive'] or 'density' in kwargs['derive']:
                mif += ('Schedule \"{}\" archive '
                        'Step 1'.format(kwargs['derive']))
            else:
                mif += 'Schedule DataTable table Stage 1\n'
        else:
            mif += 'Schedule DataTable table Stage 1\n'
            mif += 'Schedule Oxs_TimeDriver::Magnetization mags Stage 1'

        return mif

    def _checkargs(self, **kwargs):
        if 'derive' not in kwargs:
            if kwargs['t'] <= 0:
                msg = 'Positive simulation time expected (t>0).'
                raise ValueError(msg)
            if kwargs['n'] <= 0 or not isinstance(kwargs['n'], int):
                msg = 'Positive integer number of steps expected (n>0)'
                raise ValueError(msg)


"""
mif = '# SpinTransferTorqueEvolver\n'
            mif += 'Specify Anv_SpinTEvolve {\n'
            mif += '  do_precess 1\n'
            mif += '  gamma_G {}\n'.format(gamma)
            mif += '  alpha {}\n'.format(alpha)
            mif += '  method rkf54s\n'
            mif += '  u {\n'
            mif += '    Oxs_UniformScalarField {\n'
            mif += '      value {}\n'.format(u[0])
            mif += '    }\n'
            mif += '  }\n'
            mif += '  beta {}\n'.format(beta)
            mif += '}\n\n'
            evolver = 'Anv_SpinTEvolve'


Specify Anv_SpinTEvolve {
  do_precess 1
  gamma_LL 2.21e5
  method rkf54s
  alpha 0.005
  fixed_spins {
  	atlas fixed
  }
  u {Oxs_UniformScalarField {
   value 100
	}}
  beta 0.04
  	
}
"""



import os
import shutil
import numpy as np
import oommfc as oc
import discretisedfield as df


class TestMesh:
    def setup(self):
        self.p1 = (-7e-9, -5e-9, -4e-9)
        self.p2 = (7e-9, 5e-9, 4e-9)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.pbc = 'xyz'
        self.regions = {'r1': df.Region(p1=(-7e-9, -5e-9, -4e-9),
                                        p2=(7e-9, 0, 4e-9)),
                        'r2': df.Region(p1=(-7e-9, 0, -4e-9),
                                        p2=(7e-9, 2e-9, 4e-9)),
                        'r3': df.Region(p1=(-7e-9, 2e-9, -4e-9),
                                        p2=(7e-9, 5e-9, 4e-9))}

    def test_single_nopbc(self):
        name = 'mh_single_nopbc'
        if os.path.exists(name):
            shutil.rmtree(name)

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell)

        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(1, 0, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        if os.path.exists(name):
            shutil.rmtree(name)

    def test_multi_nopbc(self):
        name = 'mh_multi_nopbc'
        if os.path.exists(name):
            shutil.rmtree(name)

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2,
                       cell=self.cell, regions=self.regions)

        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(1, 0, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        if os.path.exists(name):
            shutil.rmtree(name)

    def test_single_pbc(self):
        name = 'mh_single_pbc'
        if os.path.exists(name):
            shutil.rmtree(name)

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell, pbc=self.pbc)

        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(1, 0, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        if os.path.exists(name):
            shutil.rmtree(name)

    def test_multi_pbc(self):
        name = 'mh_multi_pbc'
        if os.path.exists(name):
            shutil.rmtree(name)

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell,
                       pbc=self.pbc, regions=self.regions)

        system = oc.System(name=name)
        system.hamiltonian = oc.Zeeman(H=H)
        system.m = df.Field(mesh, dim=3, value=(1, 0, 0), norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        value = system.m(mesh.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        if os.path.exists(name):
            shutil.rmtree(name)



import oommfc.util as ou
import micromagneticmodel as mm


class CubicAnisotropy(mm.CubicAnisotropy):
    """Cubic anisotropy energy term.

    This energy term models cubic anisotropy term, defined by the
    anisotropy constant `K1` and the cubic anisotropy axes `u1` and
    `u2`. `u3` is then computed as a cross product of `u1` and `u2`.

    .. math::

           w_{ca} = -K_{1} [(\\mathbf{m} \\cdot
           \\mathbf{u}_{1})^{2}(\\mathbf{m} \\cdot
           \\mathbf{u}_{2})^{2} + (\\mathbf{m} \\cdot
           \\mathbf{u}_{2})^{2}(\\mathbf{m} \\cdot
           \\mathbf{u}_{3})^{2} + (\\mathbf{m} \\cdot
           \\mathbf{u}_{1})^{2}(\\mathbf{m} \\cdot
           \\mathbf{u}_{3})^{2}]

    `K1` is a scalar, given in J/m3, whereas `u1` and `u2` are
    three-dimensional vectors with real component values and no
    units. The axis directions must be non-zero at each point in the
    mesh, and will be normalized to unit magnitude before being used.

    Both the scalar parameter `K1` and the vector parameters `u1` and
    `u2` can be either contant is space or spatially varying. If they
    are constant, a single value is passed, e.g. `K1 = 1e5`, `u1 = (0,
    0, 1)`, and `u2 = (0, 1, 0)`. On the other hand, if one of them
    varies in space, there are two ways how they can be defined. The
    first one is using dictionaries, where the keys must be the same
    as the names of regions used when the mesh was defined
    (`discretisedfield.Mesh.regions`) and the values are single scalar
    or vector values. For instance, `K1 = {'region1': 1e5, 'region2':
    0}` and `u1 = {'region1': (0, 0, 1), 'region2': (1, 0,
    0)}`, etc. Another way of defining a spatially varying parameter is by
    passing a `discretisedfield.Field` object. The field must be
    defined on the same mesh as the micromagnetic system and it must
    have an appropriate dimension (`dim=1` for `K1` and `dim=3` for
    `u1` and `u2`).

    Parameters
    ----------
    K1 : numbers.Real, dict, discretisedfield.Field
      The cubic anisotropy constant
    u1, u2 : array_like, dict, discretisedfield.Field
      The cubic anisotropy axis
    name: str (optional)
      Name of the energy term object. Defaults to `'cubicanisotropy'`

    Examples
    --------
    1. Defining spatially constant cubic anisotropy

    >>> import oommfc as oc
    ...
    >>> K1 = 1e5
    >>> u1 = (0, 0, 1)
    >>> u2 = (0, 1, 0)
    >>> ca = oc.CubicAnisotropy(K1=K1, u1=u1, u2=u2)

    2. Defining spatially varying cubic anisotropy using dictionary

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> regions = {'region1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 4e-9)),
    ...            'region1': df.Region(p1=(0, 0, 4e-9), p2=(5e-9, 5e-9, 8e-9))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)
    ...
    >>> K1 = {'region1': 1e5, 'region2': 1e3}
    >>> u1 = (0, 0, 1)
    >>> u2 = (0, 1, 0)
    >>> ca = oc.CubicAnisotropy(K1=K1, u1=u1, u2=u2)

    3. Defining spatially varying cubic anisotropy using a
    `discretisedfield.Field` object.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n)
    >>> def u1_value(pos):
    ...     x, y, z = pos
    ...     if z <= 4e-9:
    ...         return (1, 0, 0)
    ...     else:
    ...         return (0, 0, 1)
    ...
    >>> def u2_value(pos):
    ...     x, y, z = pos
    ...     if z <= 4e-9:
    ...         return (0, 1, 0)
    ...     else:
    ...         return (1, 0, 0)
    ...
    >>> K1 = 1e5
    >>> u1 = df.Field(mesh, dim=3, value=u1_value)
    >>> u2 = df.Field(mesh, dim=3, value=u2_value)
    >>> ca = oc.CubicAnisotropy(K1=K1, u1=u1, u2=u2)

    """
    @property
    def _script(self):
        k1mif, k1name = ou.setup_scalar_parameter(self.K1, 'ca_K1')
        u1mif, u1name = ou.setup_vector_parameter(self.u1, 'ca_u1')
        u2mif, u2name = ou.setup_vector_parameter(self.u2, 'ca_u2')

        mif = ''
        mif += k1mif
        mif += u1mif
        mif += u2mif
        mif += '# CubicAnisotropy\n'
        mif += 'Specify Oxs_CubicAnisotropy {\n'
        mif += f'  K1 {k1name}\n'
        mif += f'  axis1 {u1name}\n'
        mif += f'  axis2 {u2name}\n'
        mif += '}\n\n'

        return mif



import os
import sys
import shutil
import random
import numpy as np
import oommfc as oc
import discretisedfield as df


class TestDMI:
    def setup(self):
        self.p1 = (-100e-9, 0, 0)
        self.p2 = (100e-9, 1e-9, 1e-9)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.regions = {'r1': df.Region(p1=(-100e-9, 0, 0), p2=(0, 1e-9, 1e-9)),
                        'r2': df.Region(p1=(0, 0, 0), p2=(100e-9, 1e-9, 1e-9))}

    def test_scalar(self):
        name = 'dm_scalar'
        if os.path.exists(name):
            shutil.rmtree(name)

        D = 1e-3
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell)

        system = oc.System(name=name)
        system.hamiltonian = oc.DMI(D=D, crystalclass='Cnv')

        def m_value(pos):
            return [2*random.random()-1 for i in range(3)]

        system.m = df.Field(mesh, dim=3, value=m_value, norm=Ms)
        
        md = oc.MinDriver()
        md.drive(system)

        # There are 4N cells in the mesh. Because of that the average
        # should be 0.
        assert np.linalg.norm(system.m.average) < 1

        system.delete()

    def test_dict(self):
        name = 'dm_dict'
        if os.path.exists(name):
            shutil.rmtree(name)

        D = {'r1': 0, 'r2': 1e-3}
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell,
                       regions=self.regions)

        system = oc.System(name=name)
        system.hamiltonian = oc.DMI(D=D, crystalclass='Cnv')

        def m_value(pos):
            return [2*random.random()-1 for i in range(3)]

        system.m = df.Field(mesh, dim=3, value=m_value, norm=Ms)
        
        md = oc.MinDriver()
        md.drive(system)

        r1_mesh = df.Mesh(p1=self.regions['r1'].pmin, p2=self.regions['r1'].pmax,
                          cell=self.cell)
        r2_mesh = df.Mesh(p1=self.regions['r2'].pmin, p2=self.regions['r2'].pmax,
                          cell=self.cell)
        r1_field = df.Field(r1_mesh, dim=3, value=system.m)
        r2_field = df.Field(r2_mesh, dim=3, value=system.m)
        
        assert np.linalg.norm(r1_field.average) > 1
        # There are 4N cells in the region with D!=0. Because of that
        # the average should be 0.
        assert np.linalg.norm(r2_field.average) < 1

        system.delete()

    def test_crystalclass(self):
        name = 'dm_crystalclass'

        D = 1e-3
        Ms = 1e6

        mesh = oc.Mesh(p1=self.p1, p2=self.p2, cell=self.cell)

        def m_value(pos):
            return [2*random.random()-1 for i in range(3)]
        
        for crystalclass in ['Cnv', 'T', 'O', 'D2d']:
            if crystalclass != 'Cnv' and sys.platform == 'win32':
                pass
            else:
                if os.path.exists(name):
                    shutil.rmtree(name)

                system = oc.System(name=name)
                system.hamiltonian = oc.DMI(D=D, crystalclass=crystalclass)

                system.m = df.Field(mesh, dim=3, value=m_value, norm=Ms)
        
                md = oc.MinDriver()
                md.drive(system)

                # There are 4N cells in the mesh. Because of that the
                # average should be 0.
                assert np.linalg.norm(system.m.average) < 1

                system.delete()



import oommfc.util as ou
import micromagneticmodel as mm


class RungeKuttaEvolver(mm.Evolver):
    """Runge-Kutta evolver.
    
    This class is used for collecting additional parameters, which
    cannot be extracted from the dynamics equation, but could be
    passed to `Oxs_RungeKuttaEvolve`. Only parameters which are
    defined in `_allowed_kwargs` can be passed.

    Examples
    --------
    1. Defining evolver

    >>> import oommfc as oc
    ...
    >>> evolver = oc.RungeKuttaEvolver(method='rkf54s')

    2. Passing an argument which is not allowed

    >>> import oommfc as oc
    ...
    >>> evolver = oc.RungeKuttaEvolver(myarg=3)
    Traceback (most recent call last):
       ...
    AttributeError: ...

    """
    _allowed_kwargs = ['alpha',
                       'gamma_LL',
                       'gamma_G',
                       'do_precess',
                       'allow_signed_gamma',
                       'min_timestep',
                       'max_timestep',
                       'start_dm',
                       'start_dt',
                       'stage_start',
                       'error_rate',
                       'absolute_step_error',
                       'relative_step_error',
                       'energy_precision',
                       'min_step_headroom',
                       'max_step_headroom',
                       'reject_goal',
                       'method']

    @property
    def _script(self):
        # Prepare spatially varying fields.
        mif = ''
        if hasattr(self, 'gamma_G'):
            gammamif, gammaname = ou.setup_scalar_parameter(self.gamma_G, 'pr_gamma')
            self.gamma_G = gammaname
            mif += gammamif
        if hasattr(self, 'alpha'):
            alphamif, alphaname = ou.setup_scalar_parameter(self.alpha, 'dp_alpha')
            self.alpha = alphaname
            mif += alphamif

        mif += '# RungeKuttaEvolver\n'
        mif += 'Specify Oxs_RungeKuttaEvolve:evolver {\n'
        for kwarg in self._allowed_kwargs:
            if hasattr(self, kwarg):
                mif += f'  {kwarg} {getattr(self, kwarg)}\n'
        mif += '}\n\n'

        return mif



import os
import time
import pytest
import shutil
import oommfc as oc

@pytest.mark.oommf
@pytest.mark.travis
def test_performance():
    """Test that runs an OOMMF simulation that basically doesn't do much:
    it computes the time development of a macrospin for a very short time.
    We can use this to test the overhead of starting up OOMMF.

    Running the test on 20 March 2017 on Travis (see https://travis-ci.org/joommf/oommfc/builds/212906740)
    gave this performance:

    	2017/3/20 8:49: Calling OOMMF (example-macrospin/example-macrospin.mif) ... [0.4s]
    	<1> mmarchive killed
    	<2> mmarchive killed
    	<3> mmarchive killed
    	Duration of calling OOMMF through oommfc: 1.067s
    	oommfc.oommf.status(): {'docker': True, 'host': True}
    	Start: "/usr/local/oommfc/example-macrospin/example-macrospin.mif"
    	Options: -exitondone 1 -threads 2
    	Boxsi version 1.2.1.0
    	Running on: 9bedd4e404e5
    	OS/machine: Linux/x86_64
    	User: root	PID: 13914
    	Number of threads: 2
    	Mesh geometry: 1 x 1 x 1 = 1 cells
    	Checkpoint file: /usr/local/oommfc/example-macrospin/example-macrospin.restart
    	Boxsi run end.
    	==========================================
    	Duration of calling through os.system OOMMF: 1.32s
    	Difference between oommfc and os.system call is -0.2536s = -19.21%
    	==================== 2 failed, 128 passed in 267.38 seconds ====================

    This means: the actual call of OOMMF took 0.4s (reported by our
    code), the overhead of doing this through OOMMFC meant that this
    acumulated to 1.067s. It is not clear where this overhead comes
    from and why it should be that much.

    Calling OOMMF directly through os.system took 1.32s. One would
    expect that this is a similar number to the 0.4s, so again some
    questions. Of course the above is only one data point, and could
    depend on load fluctuations on the Travis CI infrastructure.

    In any case, this shows that overheads of the order of 1 second
    are reasonable, and shouldn't be exceeded.

    Hans, Dresden, 20 March 2017

    """
    duration, mifpath = oc.test_oommf_overhead()

    # now do the same thing 'more directly':
    start = time.time()
    retvalue = os.system("tclsh $OOMMFTCL boxsi +fg " +
                         "{} -exitondone 1".format(mifpath))
    stop = time.time()
    # did the exeuction complete correctly?
    assert retvalue == 0, "Something went wrong calling OOMMF directly."

    print("==========================================")
    duration2 = stop - start
    print("Duration of calling through os.system OOMMF: {:.4}s".format(duration2))
    # use the direct call as the base line (that's duration2)
    difference = duration - duration2
    reldiff = difference / duration2
    print("Difference between oommfc and os.system call is {:.4}s = {:.4}%".format(
        difference, reldiff * 100))

    # Also write output to file (might be useful to get the data if the test passes)
    with open('travis_test_performance_summary.txt', 'tw') as f:
        f.write("=================================================================\n")
        f.write("Performance test oommfc/tests/travis_test_performance.py results:\n")
        f.write("\n")
        f.write("Call through oommfc:    {:.4}s\n".format(duration))
        f.write("Call through os.system: {:.4}s\n".format(duration2))
        f.write("Difference:             {:.4}s\n".format(difference))
        f.write("Relative difference:    {:.4}%\n".format(reldiff*100))
        f.write("-----------------------------------------------------------------\n")
        f.write("This data is from {}.\n".format(time.asctime()))
        f.write("-----------------------------------------------------------------\n")

    # Now onto the testing. These may have to be fine tuned, depending
    # on how reproducible the hardware load on the Travis system is.

    # Calling OOMMF should not take 6 seconds (as we have seen on some
    # tests this week). Let's say this should be doable in 2,
    # hopefully this also works on travis.
    assert duration2 <= 2.0, "{} should be < 2 seconds".format(duration2)

    # Here we could add a test criterion, that requires that
    # we only have a 100% overhead. This is maybe too forgiving - we should aim
    # to reduce this .
    assert abs(reldiff) < 1.0, \
        "Actual absdiff={:.4}s, reldiff={:.4} > 0.5".format(difference, reldiff)

    # The overhead should be under one second. Again, we shoud aim
    # towards a harsher criterion in due course.
    assert difference < 1.0, \
        "overhead from OOMMFC is {:.4}s and exceeding 1second".format(difference)

    shutil.rmtree("example-macrospin")

if __name__ == "__main__":
    test_performance()



import micromagneticmodel as mm


class Precession(mm.Precession):
    """Precession dynamics term.

    This dynamics term models the precession in the
    Landau-Lifshitz-Gilbert equation. It is defined by the
    gyromagnetic ratio `gamma`. `gamma` is a scalar value with m/As
    units.

    .. math::

           \\frac{\\text{d}\\mathbf{m}}{\\text{d}t} = -\\gamma
           \\mathbf{m} \\times \\mathbf{H}_\\text{eff}

    `gamma` can be either contant in space or spatially varying. If it
    is constant, a single value is passed, e.g. `gamma = 2.211e5`. On
    the other hand, if it varies in space, there are two ways how that
    can be defined. The first one is using a dictionary, where the
    keys must be the same as the names of regions used when the mesh
    was defined (`discretisedfield.Mesh.regions`) and the values are
    single scalar values. For instance, `gamma = {'region1': 2.211e5,
    'region2': 0}.  Another way of defining a spatially varying
    parameter is by passing a `discretisedfield.Field` object. The
    field must be defined on the same mesh as the micromagnetic system
    and it must have an appropriate dimension `dim=1`.

    Parameters
    ----------
    gamma : number.Real, dict, discretisedfield.Field
      Gyromagetic ratio
    name: str (optional)
      Name of the dynamics term object. Defaults to `'precession'`

    Examples
    --------
    1. Defining spatially constant gyromagnetic ratio.

    >>> import oommfc as oc
    ...
    >>> gamma = oc.consts.gamma0
    >>> pr = oc.Precession(gamma=gamma)
    >>> pr.gamma
    221276.14...

    2. Defining spatially varying gyromagnetic ratio using a
    dictionary.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> regions = {'region1': df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 4e-9)),
    ...            'region1': df.Region(p1=(0, 0, 4e-9), p2=(5e-9, 5e-9, 8e-9))}
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n, regions=regions)
    ...
    >>> gamma = {'region1': oc.consts.gamma0, 'region2': 0}
    >>> pr = oc.Precession(gamma=gamma)

    3. Defining spatially varying gyromagnetic ratio using a
    `discretisedfield.Field` object.

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (5e-9, 5e-9, 8e-9)
    >>> n = (5, 5, 8)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n)
    >>> def gamma_value(pos):
    ...     x, y, z = pos
    ...     if z <= 4e-9:
    ...         return 0
    ...     else:
    ...         return 2.211e5
    ...
    >>> gamma = df.Field(mesh, dim=3, value=gamma_value)
    >>> pr = oc.Precession(gamma=gamma)

    """
    pass



import micromagneticmodel as mm


class CGEvolver(mm.Evolver):
    """Conjugate-Gradient evolver.
    
    This class is used for collecting additional parameters, which
    cannot be extracted from the dynamics equation, but could be
    passed to `Oxs_CGEvolve`. Only parameters which are defined in
    `_allowed_kwargs` can be passed.

    Examples
    --------
    1. Defining evolver

    >>> import oommfc as oc
    ...
    >>> evolver = oc.CGEvolver(method='Polak-Ribiere')

    2. Passing an argument which is not allowed

    >>> import oommfc as oc
    ...
    >>> evolver = oc.CGEvolver(myarg=3)
    Traceback (most recent call last):
       ...
    AttributeError: ...

    """
    _allowed_kwargs = ['gradient_reset_angle',
                       'gradient_reset_count',
                       'minimum_bracket_step',
                       'maximum_bracket_step',
                       'line_minimum_angle_precision',
                       'line_minimum_relwidth',
                       'energy_precision',
                       'method']

    @property
    def _script(self):
        mif = '# CGEvolver\n'
        mif += 'Specify Oxs_CGEvolve:evolver {\n'
        for kwarg in self._allowed_kwargs:
            if hasattr(self, kwarg):
                mif += f'  {kwarg} {getattr(self, kwarg)}\n'
        mif += '}\n\n'

        return mif



import os
import glob
import json
import shutil
import datetime
import oommfc as oc
import ubermagtable as ut
import discretisedfield as df
import micromagneticmodel as mm


class Driver(mm.Driver):
    def drive(self, system, overwrite=False, **kwargs):
        """Drives the system object.

        This method takes a `oommfc.System` object and drives in the
        phase space. If `overwrite=True` is passed then the directory
        with all previously created files will be deleted before the
        run is executed. This method accepts any other arguments that
        could be required by the specific driver. After the drive is
        executed, the magnetisation and datatable of the system's
        object will be updated.

        Parameters
        ----------
        system : oommfc.System
          System object to be driven
        overwrite : bool
          If True, then the previously created files will be deleted.

        """
        # This method is implemented in the derived class.
        self._checkargs(**kwargs)

        # Generate the necessary filenames.
        dirname = os.path.join(system.name, f'drive-{system.drive_number}')
        miffilename = f'{system.name}.mif'
        jsonfilename = 'info.json'

        # Check whether a directory with the same name as system.name
        # already exists. If it does, warn the user and tell him that
        # he should pass overwrite=True to the drive method.
        if os.path.exists(dirname):
            if overwrite:
                shutil.rmtree(system.name)
            else:
                msg = (f'Directory with name={dirname} already exists. '
                       'If you want to overwrite it, pass overwrite=True '
                       'to the drive method. Otherwise, change the name '
                       'of the system or delete the directory by running '
                       'system.delete().')
                raise FileExistsError(msg)

        # Make a directory inside which OOMMF will be run.
        if not os.path.exists(dirname):
            os.makedirs(dirname)

        # Change directory to dirname
        cwd = os.getcwd()
        os.chdir(dirname)

        # Generate and save mif file.
        mif = '# MIF 2.2\n\n'
        # Output options
        mif += 'SetOptions {\n'
        mif += f'  basename {system.name}\n'
        mif += '  scalar_output_format %.12g\n'
        mif += '  scalar_field_output_format {text %#.15g}\n'
        mif += '  vector_field_output_format {text %#.15g}\n'
        mif += '}\n\n'
        mif += system._script
        mif += self._script(system, **kwargs)
        with open(miffilename, 'w') as miffile:
            miffile.write(mif)

        # Create json info file.
        info = {}
        info['drive_number'] = system.drive_number
        info['date'] = datetime.datetime.now().strftime('%Y-%m-%d')
        info['time'] = datetime.datetime.now().strftime('%H:%M:%S')
        info['driver'] = self.__class__.__name__
        info['args'] = kwargs
        with open(jsonfilename, 'w') as jsonfile:
            jsonfile.write(json.dumps(info))

        # Run OOMMF.
        oommf = oc.oommf.get_oommf_runner()
        oommf.call(argstr=miffilename)

        # Update system's m and dt attributes if the derivation of E,
        # Heff, or energy density was not asked.
        if 'derive' not in kwargs:
            # Update system's magnetisation. An example .omf filename:
            # test_sample-Oxs_TimeDriver-Magnetization-01-0000008.omf
            omffiles = glob.iglob(f'{system.name}*.omf')
            lastomffile = sorted(omffiles)[-1]
            m_field = df.Field.fromfile(lastomffile)

            # This line exists because the mesh generated in
            # df.Field.fromfile method comes from the discretisedfield
            # module where the _script method is not implemented.
            m_field.mesh = system.m.mesh
            system.m = m_field

            # Update system's datatable.
            system.dt = ut.read(f'{system.name}.odt')

        # Change directory back to cwd.
        os.chdir(cwd)

        # Increase the system's drive_number counter.
        system.drive_number += 1

    def _checkargs(self, **kwargs):
        raise NotImplementedError



import numbers
import numpy as np
import discretisedfield as df


def mif_file_vector_field(filename, name, atlas='main_atlas'):
    mif = f'# {name} file\n'
    mif += f'Specify Oxs_FileVectorField:{name} {{\n'
    mif += f'   file {filename}\n'
    mif += f'   atlas :{atlas}\n'
    mif += '}\n\n'

    return mif


def mif_vec_mag_scalar_field(field, name):
    mif = f'# {name}\n'
    mif += f'Specify Oxs_VecMagScalarField:{name} {{\n'
    mif += f'    field :{field}\n'
    mif += '}\n\n'

    return mif


def mif_atlas_vector_field(value, name, atlas='main_atlas'):
    mif = f'# {name}\n'
    mif += f'Specify Oxs_AtlasVectorField:{name} {{\n'
    mif += f'  atlas :{atlas}\n'
    mif += '  default_value {{{} {} {}}}\n'.format(*value['default'])
    mif += '  values {\n'
    for region, val in value.items():
        if region != 'default':
            mif += '    {} {{{} {} {}}}\n'.format(region, *val)
    mif += '  }'
    mif += '}\n\n'

    return mif


def mif_atlas_scalar_field(value, name, atlas='main_atlas'):
    mif = f'# {name}\n'
    mif += f'Specify Oxs_AtlasScalarField:{name} {{\n'
    mif += f'  atlas :{atlas}\n'
    mif += '  default_value {}\n'.format(value['default'])
    mif += '  values {\n'
    for region, val in value.items():
        if region != 'default':
            mif += f'    {region} {val}\n'
    mif += '  }'
    mif += '}\n\n'

    return mif


def setup_m0(parameter, name):
    if not isinstance(parameter, df.Field) or parameter.dim != 3:
        msg = 'Parameter must be a vector (dim=3) field.'
        raise ValueError(msg)
    parameter.write(f'{name}.omf')
    mif = mif_file_vector_field(f'{name}.omf', f'{name}')
    mif += mif_vec_mag_scalar_field(f'{name}', f'{name}_norm')
    return mif, f'{name}', f'{name}_norm'


def setup_scalar_parameter(parameter, name):
    if isinstance(parameter, df.Field):
        if parameter.dim != 1:
            msg = 'Parameter must be a scalar (dim=1) field.'
            raise ValueError(msg)
        parameter.write(f'{name}.ovf', extend_scalar=True)
        mif = mif_file_vector_field(f'{name}.ovf', f'{name}')
        mif += mif_vec_mag_scalar_field(f'{name}', f'{name}_norm')
        return mif, f'{name}_norm'
    elif isinstance(parameter, dict):
        if 'default' not in parameter.keys():
            parameter['default'] = 0
        mif = mif_atlas_scalar_field(parameter, f'{name}')
        return mif, f'{name}'
    elif isinstance(parameter, numbers.Real):
        return '', f'{parameter}'


def setup_vector_parameter(parameter, name):
    if isinstance(parameter, df.Field):
        if parameter.dim != 3:
            msg = 'Parameter must be a vector (dim=3) field.'
            raise ValueError(msg)
        parameter.write(f'{name}.ovf')
        mif = mif_file_vector_field(f'{name}.ovf', f'{name}')
        return mif, f'{name}'
    elif isinstance(parameter, dict):
        if 'default' not in parameter.keys():
            parameter['default'] = (0, 0, 0)
        mif = mif_atlas_vector_field(parameter, f'{name}')
        return mif, f'{name}'
    elif isinstance(parameter, (tuple, list, np.ndarray)):
        return '', '{{{} {} {}}}'.format(*parameter)


def mif_box_atlas(pmin, pmax, name='atlas'):
    mif = f'# BoxAtlas for {name}_atlas\n'
    mif += f'Specify Oxs_BoxAtlas:{name}_atlas {{\n'
    mif += f'  xrange {{{pmin[0]} {pmax[0]}}}\n'
    mif += f'  yrange {{{pmin[1]} {pmax[1]}}}\n'
    mif += f'  zrange {{{pmin[2]} {pmax[2]}}}\n'
    mif += f'  name {name}\n'
    mif += '}\n\n'

    return mif



import os
import re
import glob
import json
import shutil
import pytest
import oommfc as oc
import discretisedfield as df
import pytest


@pytest.mark.oommf
def test_info_file():
    name = "info_file"

    # Remove any previous simulation directories.
    if os.path.exists(name):
        shutil.rmtree(name)

    L = 30e-9   # (m)
    cell = (10e-9, 15e-9, 5e-9)  # (m)
    A = 1.3e-11  # (J/m)
    Ms = 8e5  # (A/m)
    H = (1e6, 0.0, 2e5)  # (A/m)
    gamma = 2.211e5  # (m/As)
    alpha = 0.02

    mesh = oc.Mesh(p1=(0, 0, 0), p2=(L, L, L), cell=cell)
    system = oc.System(name=name)
    system.hamiltonian = oc.Exchange(A=A) + oc.Zeeman(H=H)
    system.dynamics = oc.Precession(gamma) + oc.Damping(alpha)
    system.m = df.Field(mesh, value=(0.0, 0.25, 0.1), norm=Ms)

    # First (0) drive
    td = oc.TimeDriver()
    td.drive(system, t=25e-12, n=10)
    
    dirname = os.path.join(name, "drive-0")
    infofile = os.path.join(dirname, "info.json")
    assert os.path.exists(dirname)
    assert os.path.isfile(infofile)
    
    with open(infofile) as f:
        info = json.loads(f.read())
    assert 'drive_number' in info.keys()
    assert 'date' in info.keys()
    assert 'time' in info.keys()
    assert 'driver' in info.keys()
    assert 'args' in info.keys()

    assert info['drive_number'] == 0
    assert re.findall(r'\d{4}-\d{2}-\d{2}', info['date']) is not []
    assert re.findall(r'\d{2}:\d{2}-\d{2}', info['time']) is not []
    assert info['driver'] == 'TimeDriver'
    assert isinstance(info['args'], dict)
    assert info['args']['t'] == 25e-12
    assert info['args']['n'] == 10

    # Second (1) drive
    md = oc.MinDriver()
    md.drive(system)
    
    dirname = os.path.join(name, "drive-1")
    infofile = os.path.join(dirname, "info.json")
    assert os.path.exists(dirname)
    assert os.path.isfile(infofile)

    with open(infofile) as f:
        info = json.loads(f.read())
    assert 'drive_number' in info.keys()
    assert 'date' in info.keys()
    assert 'time' in info.keys()
    assert 'driver' in info.keys()
    assert 'args' in info.keys()

    assert info['drive_number'] == 1
    assert re.findall(r'\d{4}-\d{2}-\d{2}', info['date']) is not []
    assert re.findall(r'\d{2}:\d{2}-\d{2}', info['time']) is not []
    assert info['driver'] == 'MinDriver'
    assert isinstance(info['args'], dict)
    assert info['args'] == {}

    shutil.rmtree(name)



import micromagneticmodel as mm


class EulerEvolver(mm.Evolver):
    """Euler evolver.
    
    This class is used for collecting additional parameters, which
    cannot be extracted from the dynamics equation, but could be
    passed to `Oxs_EulerEvolve`. Only parameters which are defined in
    `_allowed_kwargs` can be passed.

    Examples
    --------
    1. Defining evolver

    >>> import oommfc as oc
    ...
    >>> evolver = oc.EulerEvolver(start_dm=0.02)

    2. Passing an argument which is not allowed

    >>> import oommfc as oc
    ...
    >>> evolver = oc.EulerEvolver(myarg=3)
    Traceback (most recent call last):
       ...
    AttributeError: ...

    """
    _allowed_kwargs = ['alpha',
                       'gamma_LL',
                       'gamma_G',
                       'do_precess',
                       'min_timestep',
                       'max_timestep',
                       'start_dm',
                       'error_rate',
                       'absolute_step_error',
                       'relative_step_error',
                       'step_headroom']

    @property
    def _script(self):
        mif = '# EulerEvolver\n'
        mif += 'Specify Oxs_EulerEvolve:evolver {\n'
        for kwarg in self._allowed_kwargs:
            if hasattr(self, kwarg):
                mif += f'  {kwarg} {getattr(self, kwarg)}\n'
        mif += '}\n\n'

        return mif



import os
import shutil
import micromagneticmodel as mm


class System(mm.System):
    """System class.

    This class is used for defining a micromagnetic system. In
    order to uniquely define a micromagnetic system, the following
    three parameters must be provided:

    - Name
    - Hamiltonian
    - Dynamics equation
    - Initial magnetisation configuration

    When driven, a directory with the same name (`oommfc.system.name`)
    will be created.

    Parameters
    ----------
    hamiltonian : oommfc.Hamiltonian, optional
        Hamiltonian as a sum of different energy terms.
    dynamics : oommfc.Dynamics, optional
        Dynamics as a sum of different dynamics terms.
    m : disretisedfield.Field, optional
        Initial magnetisation configuration as a three-dimensional
        field.
    name : str
        Name of the system.

    Raises
    ------
    AttributeError
        If a keyword argument which is not in the parameter list
        is passed.

    Examples
    --------
    1. Setting a simple system class

    >>> import oommfc as oc
    >>> import discretisedfield as df
    ...
    >>> p1 = (0, 0, 0)
    >>> p2 = (10e-9, 10e-9, 10e-9)
    >>> n = (5, 5, 5)
    >>> mesh = oc.Mesh(p1=p1, p2=p2, n=n)
    ...
    >>> system = oc.System(name='mysystem')
    >>> system.m = df.Field(mesh, dim=3, value=(0, 0, 1), norm=1e6)
    >>> system.hamiltonian = oc.Exchange(A=1e-11) + oc.Demag()
    >>> system.dynamics = oc.Precession(gamma=mm.consts.gamma0) + \
            oc.Damping(alpha=0.1)

    """
    def total_energy(self):
        """Total energy of the system.

        If the system was already driven, this method returns its
        energy by reading it from the datatable `system.dt`.

        """
        return self.dt.tail(1)['E'][0]

    def delete(self):
        """Delete all system related files.

        When a system object is driven using a Driver, a directory
        with `system.name` name is created where different run files
        are created. This function deletes those files if they exist.

        """
        if os.path.exists(f'{self.name}'):
            shutil.rmtree(self.name)

    @property
    def _script(self):
        mif = self.m.mesh._script
        mif += self.hamiltonian._script

        return mif



import os
import sys
import pytest
import shutil
import oommfc.oommf as oo


def check_runner(oommf_runner):
    dirname = os.path.join(os.path.dirname(__file__), 'test_files')
    os.chdir(dirname)
    argstr = 'test_oommf.mif'
    res = oommf_runner.call(argstr)
    version = oommf_runner.version()
    platform = oommf_runner.platform()

    assert isinstance(version, str)
    assert len(version) is not ''
    assert isinstance(platform, str)
    assert len(platform) is not ''
    assert res.returncode == 0

    files = os.listdir(dirname)
    for f in files:
        if f.endswith(".odt"):
            os.remove(os.path.join(dirname, f))
        elif f.endswith(".omf") and f.startswith("test_oommf-Oxs"):
            os.remove(os.path.join(dirname, f))


@pytest.mark.oommf
@pytest.mark.travis
def test_tcl_oommf_runner():
    """TclOOMMFRunner runs when OOMMFTCL environment variable is set.

    """
    oommf_tcl = os.environ.get('OOMMFTCL', None)
    oommf_runner = oo.TclOOMMFRunner(oommf_tcl)
    check_runner(oommf_runner)


@pytest.mark.oommf
@pytest.mark.travis
def test_exe_oommf_runner():
    """ExeOOMMFRunner runs when callable OOMMF exists ("oommf").

    """
    oommf_exe = "oommf"
    oommf_runner = oo.ExeOOMMFRunner(oommf_exe)
    check_runner(oommf_runner)


@pytest.mark.oommf
@pytest.mark.docker
def test_docker_oommf_runner():
    """DockerOOMMFRunner runs when docker is installed.

    """
    docker_exe = "docker"
    image = 'joommf/oommf'
    oommf_runner = oo.DockerOOMMFRunner(docker_exe, image)
    check_runner(oommf_runner)

    oommf_runner = oo.get_oommf_runner(use_cache=False,
                                       envvar='wrong_name',
                                       oommf_exe='wrong_name',
                                       docker_exe='docker')
    assert isinstance(oommf_runner, oo.DockerOOMMFRunner)


@pytest.mark.oommf
@pytest.mark.travis
def test_get_right_oommf_runner():
    # TclOOMMFRunner
    oommf_runner = oo.get_oommf_runner(use_cache=False,
                                       envvar='OOMMFTCL',
                                       oommf_exe='wrong_name',
                                       docker_exe='wrong_name')
    assert isinstance(oommf_runner, oo.TclOOMMFRunner)

    # ExeOOMMFRunner
    oommf_runner = oo.get_oommf_runner(use_cache=False,
                                       envvar='wrong_name',
                                       oommf_exe='oommf',
                                       docker_exe='wrong_name')
    assert isinstance(oommf_runner, oo.ExeOOMMFRunner)

    # OOMMF cannot be found on the system.
    with pytest.raises(EnvironmentError):
        oommf_runner = oo.get_oommf_runner(use_cache=False,
                                           envvar='wrong_name',
                                           oommf_exe='wrong_name',
                                           docker_exe='wrong_name')


@pytest.mark.oommf
@pytest.mark.travis
def test_cached_oommf_runner():
    oommf_runner = oo.get_oommf_runner(use_cache=False,
                                       envvar='wrong_name',
                                       oommf_exe='oommf',
                                       docker_exe='wrong_name')
    assert isinstance(oommf_runner, oo.ExeOOMMFRunner)

    oommf_runner = oo.get_oommf_runner(use_cache=True)
    assert isinstance(oommf_runner, oo.ExeOOMMFRunner)

    oommf_runner = oo.get_oommf_runner(use_cache=True,
                                       envvar='OOMMFTCL',
                                       oommf_exe='wrong_name',
                                       docker_exe='wrong_name')
    assert isinstance(oommf_runner, oo.ExeOOMMFRunner)


@pytest.mark.oommf
def test_get_oommf_runner():
    oommf_runner = oo.get_oommf_runner(use_cache=False)
    assert isinstance(oommf_runner, oo.OOMMFRunner)
    check_runner(oommf_runner)


@pytest.mark.oommf
def test_overhead():
    assert isinstance(oo.overhead(), float)


@pytest.mark.oommf
def test_status():
    assert oo.status() == 0


@pytest.mark.oommf
def test_runtimeerror():
    oommf_runner = oo.get_oommf_runner(use_cache=False)
    with pytest.raises(RuntimeError):
        oommf_runner.call("+wrong_argstr")



import setuptools

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='oommfc',
    version='0.8.10',
    description=('Python interface to OOMMF integrated '
                 'into Jupyter notebook.'),
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://ubermag.github.io',
    author=('Marijan Beg, Ryan A. Pepper, Thomas Kluyver, '
            'and Hans Fangohr'),
    packages=setuptools.find_packages(),
    include_package_data = True,
    install_requires=['scipy',
                      'micromagneticmodel',
                      'ubermagtable'],
    classifiers=['Development Status :: 3 - Alpha',
                 'License :: OSI Approved :: BSD License',
                 'Programming Language :: Python :: 3 :: Only',
                 'Operating System :: Unix',
                 'Operating System :: MacOS',
                 'Operating System :: Microsoft :: Windows',
                 'Topic :: Scientific/Engineering :: Physics',
                 'Intended Audience :: Science/Research',
                 'Natural Language :: English']
)



import os
import glob
import oommfc as oc
import ubermagtable as ut
import discretisedfield as df
import micromagneticmodel as mm


class Data(mm.Data):
    @property
    def effective_field(self):
        _dict = {'Demag': 'Oxs_Demag::Field',
                 'Exchange': 'Oxs_UniformExchange::Field',
                 'UniaxialAnisotropy': 'Oxs_UniaxialAnisotropy::Field',
                 'Zeeman': 'Oxs_FixedZeeman::Field',
                 'Hamiltonian': 'Oxs_RungeKuttaEvolve:evolver:Total field'}

        td = oc.TimeDriver()
        td.drive(self.system, derive=_dict[self.cls])

        dirname = os.path.join(self.system.name, 'drive-{}'.format(self.system.drive_number-1))
        ohf_file = max(glob.iglob(os.path.join(dirname, '*.ohf')),
                       key=os.path.getctime)

        return df.Field.fromfile(ohf_file)

    @property
    def energy(self):
        _dict = {'Demag': 'Demag::Energy',
                 'Exchange': 'UniformExchange::Energy',
                 'UniaxialAnisotropy': 'UniaxialAnisotropy::Energy',
                 'Zeeman': 'FixedZeeman::Energy',
                 'Hamiltonian': 'RungeKuttaEvolve:evolver:Total energy'}
        td = oc.TimeDriver()
        td.drive(self.system, derive='energy')

        dirname = os.path.join(self.system.name, 'drive-{}'.format(self.system.drive_number-1))
        odt_file = max(glob.iglob(os.path.join(dirname, '*.odt')),
                       key=os.path.getctime)

        dt = ut.read(odt_file, rename=False)

        return dt[_dict[self.cls]][0]

    @property
    def energy_density(self):
        _dict = {'Demag': 'Oxs_Demag::Energy density',
                 'Exchange': 'Oxs_UniformExchange::Energy density',
                 'UniaxialAnisotropy': ('Oxs_UniaxialAnisotropy::'
                                        'Energy density'),
                 'Zeeman': 'Oxs_FixedZeeman::Energy density',
                 'Hamiltonian': ('Oxs_RungeKuttaEvolve:evolver:'
                                 'Total energy density')}

        td = oc.TimeDriver()
        td.drive(self.system, derive=_dict[self.cls])

        dirname = os.path.join(self.system.name, 'drive-{}'.format(self.system.drive_number-1))
        oef_file = max(glob.iglob(os.path.join(dirname, '*.oef')),
                       key=os.path.getctime)

        return df.Field.fromfile(oef_file)



import micromagneticmodel as mm


class Hamiltonian(mm.Hamiltonian):
    """Hamiltonian

    This class implements the sum of individual energy terms. It is
    obtained as a result of addition of two or more energy terms.

    Examples
    --------
    1. Adding an energy term to the Hamiltonian.

    >>> import oommfc as oc
    ...
    >>> hamiltonian = oc.Hamiltonian()
    >>> hamiltonian += mm.DMI(D=1e-3, crystalclass='Cnv')

    2. Creating the Hamiltoninan as a sum of two energy terms

    >>> import oommfc as oc
    ...
    >>> hamiltonian = oc.Exchange(A=1e-12) + oc.Zeeman(H=(0, 0, 1e6))

    """
    @property
    def _script(self):
        mif = ''
        for term in self.terms:
            mif += term._script

        return mif
